@@ -6,15 +6,15 @@
 #	You may distribute this file under the terms of the Artistic
 #	License, as specified in the README file.
 #
-# $Id: Mail.pm,v 1.7 2000/05/07 06:31:07 arensb Exp $
+# $Id: Mail.pm,v 1.12 2000/09/24 16:25:49 arensb Exp $
 
 use strict;
 package Palm::Mail;
 use Palm::Raw();
-use Palm::StdAppInfo;
+use Palm::StdAppInfo();
 use vars qw( $VERSION @ISA );
 
-$VERSION = (qw( $Revision: 1.7 $ ) )[1];
+$VERSION = sprintf "%d.%03d", '$Revision: 1.12 $ ' =~ m{(\d+)\.(\d+)};
 @ISA = qw( Palm::Raw Palm::StdAppInfo );
 
 =head1 NAME
@@ -167,6 +167,9 @@ sub new
 
 Creates a new Mail record, with blank values for all of the fields.
 
+C<new_Record> does B<not> add the new record to C<$pdb>. For that,
+you want C<$pdb-E<gt>append_Record>.
+
 Note: the time given by the C<year>, C<month>, C<day>, C<hour>, and
 C<minute> fields in the new record are initialized to the time when
 the record was created. They should be reset to the time when the
@@ -210,6 +213,8 @@ sub new_Record
 	$retval->{sentTo} = undef;
 
 	$retval->{body} = "";
+
+	return $retval;
 }
 
 # ParseAppInfoBlock
@@ -228,7 +233,7 @@ sub ParseAppInfoBlock
 	# Get the standard parts of the AppInfo block
 	$std_len = &Palm::StdAppInfo::parse_StdAppInfo($appinfo, $data);
 
-	$data = substr $data, $std_len;		# Remove the parsed part
+	$data = $appinfo->{other};		# Look at the non-category part
 
 	# Get the rest of the AppInfo block
 	my $unpackstr =		# Argument to unpack()
@@ -254,16 +259,16 @@ sub PackAppInfoBlock
 	my $self = shift;
 	my $retval;
 
-	# Pack the standard part of the AppInfo block
-	$retval = &Palm::StdAppInfo::pack_StdAppInfo($self->{appinfo});
-
-	# And the application-specific stuff
-	$retval .= pack "x2 n Cx N n",
+	# Pack the non-category part of the AppInfo block
+	$self->{appinfo}{other} = pack "x2 n Cx N n",
 		$self->{appinfo}{dirty_AppInfo},
 		$self->{appinfo}{sort_order},
 		$self->{appinfo}{unsent},
 		$self->{appinfo}{sig_offset};
 
+	# Pack the AppInfo block
+	$retval = &Palm::StdAppInfo::pack_StdAppInfo($self->{appinfo});
+
 	return $retval;
 }
 
@@ -362,8 +367,8 @@ sub ParseRecord
 	$record{to} = $to;
 	$record{cc} = $cc;
 	$record{bcc} = $bcc;
-	$record{reply_to} = $replyTo;
-	$record{sent_to} = $sentTo;
+	$record{replyTo} = $replyTo;
+	$record{sentTo} = $sentTo;
 	$record{body} = $body;
 	$record{extra} = $extra;
 
@@ -401,8 +406,8 @@ sub PackRecord
 		$record->{to},
 		$record->{cc},
 		$record->{bcc},
-		$record->{reply_to},
-		$record->{sent_to},
+		$record->{replyTo},
+		$record->{sentTo},
 		$record->{body};
 	$retval .= "\0";
 
